@@ -7,6 +7,8 @@ class FDSClientConfiguration(object):
 
   URI_SUFFIX = 'fds.api.xiaomi.com'
 
+  URI_NET_SUFFIX = 'fds.api.xiaomi.net'
+
   URI_CDN_SUFFIX = 'fds.api.mi-img.com'
 
   def __init__(self, region_name='cnbj0',
@@ -26,7 +28,10 @@ class FDSClientConfiguration(object):
     :param timeout: connection and read timeout (seconds)
     :param max_retries:
     """
-    self._region_name = region_name
+    if region_name is None:
+      self._region_name = 'cnbj0'
+    else:
+      self._region_name = region_name
     self._enable_cdn_for_download = enable_cdn_for_download
     self._enable_cdn_for_upload = enable_cdn_for_upload
     self._enable_https = enable_https
@@ -34,7 +39,10 @@ class FDSClientConfiguration(object):
     self._timeout = timeout
     self._max_retries = max_retries
     self._debug = False
-    self._endpoint = endpoint
+    if endpoint is None:
+      endpoint = self._region_name + '.' + self.URI_SUFFIX
+
+    self.set_endpoint(endpoint)
 
     self._threshold_size = threshold_size
     self._part_size = part_size
@@ -84,7 +92,13 @@ class FDSClientConfiguration(object):
     self._max_retries = max_retries
 
   def set_endpoint(self, endpoint):
+    assert endpoint.endswith(self.URI_SUFFIX) or endpoint.endswith(self.URI_NET_SUFFIX)
+
     self._endpoint = endpoint
+    suffix = self._endpoint[-19:]
+    i = self._endpoint.index(suffix)
+    self._region_name = self._endpoint[0:i]
+    self._cdn_endpoint = self.URI_CDN + '.' + self._region_name + '.' + self.URI_CDN_SUFFIX
 
   def get_download_base_uri(self):
     return self._build_base_uri(self._enable_cdn_for_download)
@@ -105,15 +119,10 @@ class FDSClientConfiguration(object):
     else:
       base_uri += self.URI_HTTP
 
-    region = self._region_name
-    if not region:
-      region = "cnbj0"
-    if self._endpoint is not None:
-      base_uri += self._endpoint
-    elif enable_cdn:
-      base_uri += self.URI_CDN + '.' + region + '.' + self.URI_CDN_SUFFIX
+    if enable_cdn:
+      base_uri += self._cdn_endpoint
     else:
-      base_uri += region + '.' + self.URI_SUFFIX
+      base_uri += self._endpoint
 
     base_uri += '/'
     return base_uri
