@@ -1,8 +1,11 @@
+from __future__ import print_function
+
 import concurrent.futures
 import errno
 import json
 import logging
 import os
+import mimetypes
 import sys
 from pprint import pprint
 from sys import version_info
@@ -11,13 +14,21 @@ import fire
 import time
 from datetime import datetime
 
+
 from fds import FDSClientConfiguration, GalaxyFDSClient, GalaxyFDSClientException
 from fds.fds_cli import multipart_upload_buffer_size, max_upload_retry_time
 from fds.model.upload_part_result_list import UploadPartResultList
-from fds.utils import rfc822_timestamp
+from fds.model.fds_object_metadata import FDSObjectMetadata
+from fds.utils import rfc822_timestamp, file_md5
+from fds.auth.common import Common
+from fds.model.permission import AccessControlList, UserGroups, Permission, GrantType, Grant, Grantee, Owner
+
 
 IS_PY3 = version_info[0] >= 3
 
+if not IS_PY3:
+  input = raw_input
+  
 log_format = '%(asctime)-15s [%(filename)s:%(lineno)d] %(message)s'
 logging.basicConfig(format=log_format)
 logger = logging.getLogger('fds.cli')
@@ -36,22 +47,6 @@ def mkdirs(path):
       pass
 
 
-def async_task(fds_client, upload_token, part_number, data):
-  for i in range(max_upload_retry_time):
-    try:
-      rtn = fds_client.upload_part(bucket_name=upload_token.bucket_name,
-                                   object_name=upload_token.object_name,
-                                   upload_id=upload_token.upload_id,
-                                   part_number=part_number,
-                                   data=data)
-      return rtn
-    except GalaxyFDSClientException:
-      sleep_seconds = (i + 1) * 10
-      CLIPrinter.warn("upload part %d failed, retry after %d seconds" % (
-        part_number, sleep_seconds))
-      time.sleep(sleep_seconds)
-
-
 class LocalConfig(object):
   def __init__(self):
     self.__config_path = os.path.join(
@@ -59,7 +54,12 @@ class LocalConfig(object):
     mkdirs(os.path.join(os.path.expanduser("~"), ".config", "xiaomi"))
 
     self.__data = None
-    with open(self.__config_path) as f:
+
+    if not os.path.exists(self.__config_path):
+      with open(self.__config_path, 'w+') as f:
+        f.writelines("{}")
+  
+    with open(self.__config_path, 'r+') as f:
       self.__data = json.load(f)
 
   @property
@@ -115,7 +115,6 @@ class CLIPrinter(object):
   @staticmethod
   def fail(message):
     print("[fail] ", message)
-    sys.exit(1)
 
   @staticmethod
   def print_bucket(bucket):
@@ -126,10 +125,18 @@ class CLIPrinter(object):
     CLIPrinter.fail("wrong fds url format or local not exists")
 
   @staticmethod
-  def print_object(object_name, metadata):
-    print("%s\t%s\t%s" % (metadata.metadata['last-modified'],
-                          metadata.metadata['content-length'],
-                          object_name))
+  def print_object(object_name, metadata, human):
+    content_length = int(metadata.metadata['x-xiaomi-meta-content-length'])
+    if human == 'k':
+      content_length = content_length / 8.0 / 1024
+    elif human == 'm':
+      content_length = content_length / 8.0 / 1024 / 1024
+    elif human == 'g':
+      content_length = content_length / 8.0 / 1024 / 1024 / 1024
+    print("{}\t{:.2f}{}\t{}".format(metadata.metadata['last-modified'],
+                                    content_length,
+                                    human,
+                                    object_name.encode('utf-8').strip()))
 
   @staticmethod
   def print_lifecycle(lifecycle):
@@ -151,8 +158,7 @@ class FDSURL(object):
     fds://bucket_name
     fds://bucket_name/
     """
-    tmp = self.__none_fds_prefix_name.split(self.bucket_name() + '/')
-    if len(tmp) <= 1 or (len(tmp) == 2 and tmp[1] == ''):
+    if self.__none_fds_prefix_name == self.bucket_name() or self.__none_fds_prefix_name == self.bucket_name()+"/":
       return True
     return False
 
@@ -164,8 +170,7 @@ class FDSURL(object):
       return None
     if self.is_bucket_url():
       return None
-    tmp = self.__none_fds_prefix_name.split(self.bucket_name() + '/')
-    return tmp[1]
+    return self.__none_fds_prefix_name[len(self.bucket_name())+1:]
 
   def file_name(self):
     if not self.is_object_url():
@@ -197,8 +202,7 @@ class FDSURL(object):
       return None
     if not self.is_dir():
       return None
-    tmp = self.__none_fds_prefix_name.split(self.bucket_name() + '/')
-    return tmp[1]
+    return self.__none_fds_prefix_name[len(self.bucket_name())+1:]
 
   def is_object_url(self):
     return not self.is_bucket_url() and not self.is_object_dir()
@@ -214,112 +218,6 @@ class FDSURL(object):
     return self.__url
 
 
-class GalaxyFDSClientWithExceptionHandler(GalaxyFDSClient):
-  def __init__(self, access_key=None, access_secret=None, config=None):
-    super().__init__(access_key, access_secret, config)
-
-  def delete_bucket(self, bucket_name, force=False):
-    try:
-      if force:
-        all_objects = self.list_all_objects(bucket_name, '', '')
-        names = []
-        for o in all_objects:
-          names.append(o.object_name)
-        self.delete_objects(bucket_name, names)
-
-      return super().delete_bucket(bucket_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def does_bucket_exist(self, bucket_name):
-    try:
-      return super().does_bucket_exist(bucket_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def create_bucket(self, bucket_name):
-    try:
-      super().create_bucket(bucket_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def list_authorized_buckets(self):
-    try:
-      return super().list_authorized_buckets()
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def list_next_batch_of_objects(self, previous):
-    try:
-      return super().list_next_batch_of_objects(previous)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def put_object(self, bucket_name, object_name, data, metadata=None):
-    try:
-      return super().put_object(bucket_name, object_name, data, metadata)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def delete_object(self, bucket_name, object_name, **kwargs):
-    try:
-      return super().delete_object(bucket_name, object_name, **kwargs)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def copy_object(self, src_bucket_name, src_object_name, dst_bucket_name, dst_object_name):
-    try:
-      return super().copy_object(src_bucket_name, src_object_name, dst_bucket_name, dst_object_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def get_object(self, bucket_name, object_name, position=0, size=4096, stream=None,
-                 version_id=None):
-    try:
-      return super().get_object(bucket_name, object_name, position, size, stream, version_id)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def list_objects(self, bucket_name, prefix='', delimiter=None, max_keys=None):
-    try:
-      return super().list_objects(bucket_name, prefix, delimiter, max_keys)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def get_object_metadata(self, bucket_name, object_name):
-    try:
-      return super().get_object_metadata(bucket_name, object_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def does_object_exists(self, bucket_name, object_name):
-    try:
-      return super().does_object_exists(bucket_name, object_name)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def delete_objects(self, bucket_name, object_names):
-    try:
-      return super().delete_objects(bucket_name, object_names)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def generate_presigned_uri(self, base_uri, bucket_name, object_name, expiration,
-                             http_method="GET",
-                             content_type=None, sub_resources=None):
-    try:
-      return super().generate_presigned_uri(base_uri, bucket_name, object_name,
-                                            expiration, http_method, content_type, sub_resources)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-  def get_lifecycle_config(self, bucket_name, rule_id=None):
-    try:
-      return super().get_lifecycle_config(bucket_name, rule_id)
-    except GalaxyFDSClientException as e:
-      CLIPrinter.fail(e.args)
-
-
 class FDSCli(object):
   """
   Advanced fds cli you deserved!
@@ -336,19 +234,17 @@ class FDSCli(object):
     if self._sk is None:
       self.config()
     self._endpoint = self._local_config.endpoint if endpoint is None else endpoint
-    if self._sk is None:
+    if self._endpoint is None:
       self.config()
 
-    logger.debug("endpoint: " + self._endpoint)
-
     self._fds_config = FDSClientConfiguration(region_name="awsde0",
                                               enable_https=False,
                                               enable_cdn_for_download=False,
                                               enable_cdn_for_upload=False,
                                               endpoint=self._endpoint)
-    self._fds = GalaxyFDSClientWithExceptionHandler(access_key=self._ak,
-                                                    access_secret=self._sk,
-                                                    config=self._fds_config)
+    self._fds = GalaxyFDSClient(access_key=self._ak,
+                                access_secret=self._sk,
+                                config=self._fds_config)
 
   def config(self):
     """
@@ -381,7 +277,11 @@ class FDSCli(object):
     if not url.is_bucket_url():
       CLIPrinter.wrong_format()
     bucket_name = url.bucket_name()
-    self._fds.create_bucket(bucket_name)
+    try:
+      self._fds.create_bucket(bucket_name)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
     CLIPrinter.done("create bucket [%s]" % bucket_name)
 
   def rb(self, fds_url, force=False):
@@ -393,8 +293,32 @@ class FDSCli(object):
     url = FDSURL(fds_url)
     if not url.is_bucket_url():
       CLIPrinter.wrong_format()
+      return
     bucket_name = url.bucket_name()
-    self._fds.delete_bucket(bucket_name, force)
+    if force:
+      result = self._fds.list_objects(bucket_name, '', '')
+      while True:
+        names = []
+        try:
+          for object_summary in result.objects:
+            names.append(object_summary.object_name)
+        except GalaxyFDSClientException as e:
+          CLIPrinter.warn(e.message)
+        try:
+          self._fds.delete_objects(bucket_name, names)
+        except GalaxyFDSClientException as e:
+          CLIPrinter.fail(e.message)
+          return
+        if result.is_truncated:
+          result = self._fds.list_next_batch_of_objects(result)
+        else:
+          break
+
+    try:
+      self._fds.delete_bucket(bucket_name)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
     CLIPrinter.done("remove bucket [%s]" % bucket_name)
 
   def rm(self, fds_url):
@@ -407,22 +331,35 @@ class FDSCli(object):
 
     if url.is_bucket_url():
       CLIPrinter.fail("please enter a object resource address to remove")
+      return
     object_name = url.object_name()
 
-    self._fds.delete_object(bucket_name, object_name)
+    try:
+      self._fds.delete_object(bucket_name, object_name)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
     CLIPrinter.done("remove object: [%s] in bucket [%s]" % (
       object_name, bucket_name))
 
-  def ls(self, fds_url=None, recursive=False):
+  def ls(self, fds_url=None, recursive=False, human='k'):
     """
     list all buckets or objects in a bucket
     :param fds_url: fds url format like fds://bucket
     :param recursive: recursive listing
     """
 
+    if human != 'k' and human != 'm' and human != 'g':
+      CLIPrinter.fail("human should be in 'k|m|g'")
+      return
     # bucket_url is None means listing all bucket name
     if fds_url is None:
-      buckets = self._fds.list_authorized_buckets()
+      buckets = None
+      try:
+        buckets = self._fds.list_authorized_buckets()
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
+        return
       for bucket in buckets:
         CLIPrinter.print_bucket(bucket)
     else:
@@ -438,17 +375,24 @@ class FDSCli(object):
           prefix = url.object_dir()
         results = self._fds.list_all_objects(
           bucket_name, prefix, delimiter)
-        for result in results:
-          o = self._fds.get_object(
-            bucket_name, result.object_name)
-          CLIPrinter.print_object(o.summary.object_name, o.metadata)
+        try:
+          for result in results:
+            metadata = self._fds.get_object_metadata(bucket_name, result.object_name)
+            CLIPrinter.print_object(result.object_name, metadata, human)
+        except GalaxyFDSClientException as e:
+          CLIPrinter.fail(e.message)
       else:
         object_name = url.object_name()
-        o = self._fds.get_object(bucket_name, object_name)
-        if o is not None:
-          CLIPrinter.print_object(o.summary.object_name, o.metadata)
+        metadata = None
+        try:
+          metadata = self._fds.get_object_metadata(bucket_name, object_name)
+        except GalaxyFDSClientException as e:
+          CLIPrinter.fail(e.message)
+          return
+        if metadata is not None:
+          CLIPrinter.print_object(object_name, metadata, human)
 
-  def presigned(self, fds_url, expires=1):
+  def presigned(self, fds_url, expires=1, cdn=False):
     """
     presigned command generates presigned url for download project
     :param fds_url: format url like fds://bucket/a.txt
@@ -468,11 +412,16 @@ class FDSCli(object):
                              float((datetime.utcnow() - datetime(1970, 1, 1, 0, 0, 0,
                                                                  0)).total_seconds())))
 
-    CLIPrinter.done(
-      'generated presigned url: ' + self._fds.generate_presigned_uri(
-        self._fds_config.get_base_uri(), bucket_name,
-        object_name,
-        expiration))
+    try:
+      if cdn:
+        base_uri = self._fds_config.get_cdn_base_uri()
+      else:
+        base_uri = self._fds_config.get_base_uri()
+      u = self._fds.generate_presigned_uri(base_uri, bucket_name, object_name, expiration)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
+    CLIPrinter.done('generated presigned url: ' + u)
 
   def ttl(self, fds_url):
     """
@@ -481,7 +430,11 @@ class FDSCli(object):
     """
     url = FDSURL(fds_url)
     bucket_name = url.bucket_name()
-    ttl = self._fds.get_lifecycle_config(bucket_name)
+    try:
+      ttl = self._fds.get_lifecycle_config(bucket_name)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
     if url.is_bucket_url():
       CLIPrinter.print_lifecycle(ttl)
     elif url.is_object_url():
@@ -496,7 +449,7 @@ class FDSCli(object):
     else:
       CLIPrinter.wrong_format()
 
-  def cp(self, src, dst, recursive=False):
+  def cp(self, src, dst, recursive=False, autodetect_mimetype=False):
     """
     cp command do lots of things.
     1. file upload
@@ -527,7 +480,7 @@ class FDSCli(object):
 
       if src_url.is_object_url():
         self._download(src_url, dst)
-      elif not src_url.is_object_dir() and os.path.isdir(dst):
+      elif src_url.is_object_dir() and os.path.isdir(dst):
         self._download_batch(src_url, dst, recursive)
       else:
         CLIPrinter.wrong_format()
@@ -536,9 +489,9 @@ class FDSCli(object):
       dst_url = FDSURL(dst)
 
       if os.path.isfile(src):
-        self._upload(src, dst_url)
+        self._upload(src, dst_url, autodetect_mimetype=autodetect_mimetype)
       elif os.path.isdir(src) and not dst_url.is_object_url():
-        self._upload_batch(src, dst_url, recursive)
+        self._upload_batch(src, dst_url, recursive, autodetect_mimetype=autodetect_mimetype)
       else:
         CLIPrinter.wrong_format()
     else:
@@ -554,7 +507,10 @@ class FDSCli(object):
       dst_object_name = dst_url.object_name()
     else:
       dst_object_name = src_object_name
-    self._fds.copy_object(src_bucket_name, src_object_name, dst_bucket_name, dst_object_name)
+    try:
+      self._fds.copy_object(src_bucket_name, src_object_name, dst_bucket_name, dst_object_name)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
     CLIPrinter.done(
       "copy %s/%s to %s/%s" % (src_bucket_name, src_object_name, dst_bucket_name, dst_object_name))
 
@@ -572,10 +528,13 @@ class FDSCli(object):
 
     all_objects = self._fds.list_all_objects(bucket_name=src_bucket_name, prefix=prefix,
                                              delimiter=delimiter)
-    for o in all_objects:
-      o_name = o.object_name
-      self._fds.copy_object(src_bucket_name, o_name, dst_bucket_name, o_name)
-      CLIPrinter.done("copy %s/%s to %s/%s" % (src_bucket_name, o_name, dst_bucket_name, o_name))
+    try:
+      for o in all_objects:
+        o_name = o.object_name
+        self._fds.copy_object(src_bucket_name, o_name, dst_bucket_name, o_name)
+        CLIPrinter.done("copy %s/%s to %s/%s" % (src_bucket_name, o_name, dst_bucket_name, o_name))
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
 
   def _download(self, src_url, dst):
     src_bucket_name = src_url.bucket_name()
@@ -593,15 +552,26 @@ class FDSCli(object):
 
     mtime = None
     if os.path.isfile(dst_name):
+      local_md5 = file_md5(dst_name)
+      remote_md5 = self._fds.get_object_metadata(src_bucket_name, src_object_name).metadata.get(Common.CONTENT_MD5)
+      if remote_md5 is not None and local_md5 == remote_md5:
+        CLIPrinter.done("download %s/%s to local(skip because of same md5)" % (src_bucket_name, src_object_name))
+        return
+
       mtime = os.path.getmtime(dst_name)
 
-    fds_object = self._fds.get_object(bucket_name=src_bucket_name, object_name=src_object_name,
-                                      stream=True)
+    try:
+      fds_object = self._fds.get_object(bucket_name=src_bucket_name, object_name=src_object_name,
+                                        stream=True)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
+      return
     lm = fds_object.metadata.metadata['last-modified']
     remote_modified = rfc822_timestamp(lm)
 
     # if last-modified of local file is not less last-modified of remote file, skip
     if mtime is not None and datetime.fromtimestamp(mtime) >= remote_modified:
+      CLIPrinter.done("download %s/%s to local(skip because of updated)" % (src_bucket_name, src_object_name))
       return
 
     length_left = IS_PY3 and sys.maxsize or sys.maxint
@@ -630,48 +600,103 @@ class FDSCli(object):
     if recursive:
       delimiter = ""
 
-    all_objects = self._fds.list_all_objects(bucket_name=src_bucket_name, prefix=prefix,
-                                             delimiter=delimiter)
-    for o in all_objects:
-      o_name = o.object_name
-      url = FDSURL(fds_prefix + src_bucket_name + "/" + o_name)
-      if url.is_object_url():
-        self._download(url, dst)
+    all_objects = self._fds.list_all_objects(bucket_name=src_bucket_name, prefix=prefix, delimiter=delimiter)
+    try:
+      for o in all_objects:
+        o_name = o.object_name
+        url = FDSURL(fds_prefix + src_bucket_name + "/" + o_name)
+        if url.is_object_url():
+          self._download(url, dst)
+    except GalaxyFDSClientException as e:
+      CLIPrinter.fail(e.message)
 
-  def _upload(self, filename, dst_url, sync=False):
+  def _upload(self, filename, dst_url, autodetect_mimetype, sync=False):
+    if not os.path.exists(filename):
+      CLIPrinter.warn("{} is a bad file".format(filename))
+      return
     dst_bucket_name = dst_url.bucket_name()
     if dst_url.is_object_url():
       dst_object_name = dst_url.object_name()
     elif sync:
       dst_object_name = filename[2:]
+    elif dst_url.is_object_dir():
+      dst_object_name = dst_url.object_dir() + os.path.basename(filename)
     else:
       dst_object_name = os.path.basename(filename)
+    try:
+      if self._fds.does_object_exists(dst_bucket_name, dst_object_name):
+        # check md5 firstly
+        metadata = self._fds.get_object_metadata(dst_bucket_name, dst_object_name)
+        if metadata.metadata.get(Common.CONTENT_MD5) is not None:
+          local_md5 = file_md5(filename)
+          if local_md5 == metadata.metadata.get(Common.CONTENT_MD5):
+            CLIPrinter.done('upload object %s/%s(skip because of same md5)' % (dst_bucket_name, dst_object_name))
+            return
 
-    if self._fds.does_object_exists(dst_bucket_name, dst_object_name):
-      mtime = None
-      if os.path.isfile(filename):
-        mtime = os.path.getmtime(filename)
+        # check last-modified
+        mtime = None
+        if os.path.isfile(filename):
+          mtime = os.path.getmtime(filename)
 
-      fds_object = self._fds.get_object(dst_bucket_name, dst_object_name)
-      lm = fds_object.metadata.metadata['last-modified']
-      remote_modified = rfc822_timestamp(lm)
+        lm = metadata.metadata[Common.LAST_MODIFIED]
+        remote_modified = rfc822_timestamp(lm)
 
-      # if last-modified of local file is not less last-modified of remote file, skip
-      if mtime is not None and datetime.fromtimestamp(mtime) <= remote_modified:
-        return
+        # if last-modified of local file is not less last-modified of remote file, skip
+        if mtime is not None and datetime.fromtimestamp(mtime) <= remote_modified:
+          CLIPrinter.done('upload object %s/%s(skip because of updated)' % (dst_bucket_name, dst_object_name))
+          return
+    except Exception as e:
+      CLIPrinter.fail(e.message)
+      return
+    mimetype = None
+    if autodetect_mimetype:
+      mimetype = mimetypes.guess_type(filename)[0]
+    metadata = FDSObjectMetadata()
+    if mimetype is not None:
+      metadata.add_header(Common.CONTENT_TYPE, mimetype)
+    result = None
 
     with open(filename, "rb") as f:
       file_length = os.path.getsize(filename)
       if file_length < multipart_upload_buffer_size:
-        result = self._fds.put_object(dst_bucket_name, dst_object_name, f, metadata=None)
+        try:
+          result = self._fds.put_object(dst_bucket_name, dst_object_name, f, metadata=metadata)
+        except GalaxyFDSClientException as e:
+          CLIPrinter.fail(e.message)
       else:
-        result = self._upload_concurrency(dst_bucket_name, dst_object_name, f)
+        try:
+          upload_token = self._fds.init_multipart_upload(dst_bucket_name, dst_object_name)
+          part_number = 1
+          result_list = []
+          while True:
+            data = f.read(multipart_upload_buffer_size)
+            if len(data) <= 0:
+              break
+            for i in range(max_upload_retry_time):
+              upload_result = None
+              try:
+                upload_result = self._fds.upload_part(dst_bucket_name, dst_object_name, upload_token.upload_id, part_number, data)
+                result_list.append(upload_result)
+                break
+              except GalaxyFDSClientException as e:
+                sleep_seconds = (i + 1) * 10
+                CLIPrinter.warn("upload part %d failed, retry after %d seconds" % (
+                  part_number, sleep_seconds))
+                time.sleep(sleep_seconds)
+            part_number = part_number + 1
+          upload_part_result = UploadPartResultList({"uploadPartResultList": result_list})
+          result = self._fds.complete_multipart_upload(upload_token.bucket_name, upload_token.object_name,
+                                                       upload_token.upload_id, metadata,
+                                                       json.dumps(upload_part_result))
+        except Exception as e:
+          self._fds.abort_multipart_upload(dst_bucket_name, dst_object_name, upload_token.upload_id)
+          CLIPrinter.fail(e.message)
     if result is not None:
       CLIPrinter.done('upload object %s/%s' % (dst_bucket_name, dst_object_name))
     else:
       CLIPrinter.fail('upload object %s/%s' % (dst_bucket_name, dst_object_name))
 
-  def _upload_batch(self, d, dst_url, recursive, sync=False):
+  def _upload_batch(self, d, dst_url, recursive, autodetect_mimetype, sync=False):
     for root, dirs, files in os.walk(d):
       relative_dir = os.path.relpath(root, d)
       if relative_dir != '.' and relative_dir != '..' and relative_dir.startswith('.'):
@@ -679,37 +704,12 @@ class FDSCli(object):
         continue
       for filename in files:
         object_name = os.path.join(root, filename)
-        self._upload(object_name, dst_url, sync)
+        object_name = '/'.join(object_name.split('\\'))
+        self._upload(object_name, dst_url, autodetect_mimetype, sync)
       if not recursive:
         break
 
-  def _upload_concurrency(self, bucket_name, object_name, stream):
-    upload_token = None
-    try:
-      upload_token = self._fds.init_multipart_upload(bucket_name, object_name)
-      part_number = 1
-      futures = []
-      result_list = []
-      with concurrent.futures.ThreadPoolExecutor(max_workers=5) as executor:
-        while True:
-          data = stream.read(multipart_upload_buffer_size)
-          if len(data) <= 0:
-            break
-          future = executor.submit(async_task, self._fds, upload_token, part_number, data)
-          futures.append(future)
-          part_number += 1
-
-      for f in concurrent.futures.as_completed(futures):
-        result_list.append(f.result())
-      upload_part_result = UploadPartResultList({"uploadPartResultList": result_list})
-      return self._fds.complete_multipart_upload(upload_token.bucket_name, upload_token.object_name,
-                                                 upload_token.upload_id, None,
-                                                 json.dumps(upload_part_result))
-    except Exception as e:
-      self._fds.abort_multipart_upload(bucket_name, object_name, upload_token.upload_id)
-      CLIPrinter.fail(e.args)
-
-  def sync(self, src, dst):
+  def sync(self, src, dst, autodetect_mimetype=False):
     """
     sync command syncs between (local directory and fds) (fds and local directory) (fds and fds)
     :param src: src can be a fds bucket url like fds://bucketname or '.'
@@ -725,22 +725,25 @@ class FDSCli(object):
 
       src_bucket_name = src_url.bucket_name()
       all_objects = self._fds.list_all_objects(bucket_name=src_bucket_name, prefix='', delimiter='')
-      for o in all_objects:
-        o_name = o.object_name
-        url = FDSURL(fds_prefix + src_bucket_name + '/' + o_name)
-        if '/' not in o_name:
-          self._download(url, dst)
-        elif url.is_object_url():
-          o_file_name = o_name.split('/')[-1]
-          o_dir = o_name.split(o_file_name)[0]
-          mkdirs(o_dir)
-          self._download(url, o_dir)
+      try:
+        for o in all_objects:
+          o_name = o.object_name
+          url = FDSURL(fds_prefix + src_bucket_name + '/' + o_name)
+          if '/' not in o_name:
+            self._download(url, dst)
+          elif url.is_object_url():
+            o_file_name = o_name.split('/')[-1]
+            o_dir = o_name.split(o_file_name)[0]
+            mkdirs(o_dir)
+            self._download(url, o_dir)
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
 
     elif not FDSURL.is_fds_url(src) and FDSURL.is_fds_url(dst):
       dst_url = FDSURL(dst)
       if not src.strip() == '.' or not dst_url.is_bucket_url():
         CLIPrinter.wrong_format()
-      self._upload_batch(src, dst_url, True, True)
+      self._upload_batch(src, dst_url, True, autodetect_mimetype, True)
 
     elif FDSURL.is_fds_url(src) and FDSURL.is_fds_url(dst):
       self.cp(src, dst)
@@ -748,6 +751,63 @@ class FDSCli(object):
     else:
       CLIPrinter.wrong_format()
 
+  def make_public(self, url):
+    if not FDSURL.is_fds_url(url):
+      CLIPrinter.wrong_format()
+      return
+    url = FDSURL(url)
+    if url.is_object_url():
+      try:
+        self._fds.set_public(url.bucket_name(), url.object_name())
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
+        return
+    elif url.is_bucket_url():
+      try:
+        acl = AccessControlList()
+        grant = Grant(Grantee(UserGroups.ALL_USERS), Permission.READ)
+        grant.type = GrantType.GROUP
+        acl.add_grant(grant)
+        self._fds.set_bucket_acl(url.bucket_name(), acl)
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
+        return
+    else: 
+      CLIPrinter.wrong_format()
+      return
+
+  def make_outside(self, url, close=False):
+    if not FDSURL.is_fds_url(url):
+      CLIPrinter.wrong_format()
+      return
+    url = FDSURL(url)
+
+    if url.is_bucket_url():
+      try:
+        if close:
+          self._fds.set_bucket_outside_access(url.bucket_name(), False)
+        else:
+          self._fds.set_bucket_outside_access(url.bucket_name(), True)
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
+        return
+    elif url.is_object_url():
+      try:
+        if close:
+          self._fds.set_object_outside_access(url.bucket_name(), url.object_name(), False)
+        else:
+          self._fds.set_object_outside_access(url.bucket_name(), url.object_name(), True)
+      except GalaxyFDSClientException as e:
+        CLIPrinter.fail(e.message)
+        return
+    else:
+      CLIPrinter.wrong_format()
+      return
+
+  def info(self):
+    print("Access Key ID: {}".format(self._local_config.ak))
+    print("Access Secret Key: {}".format(self._local_config.sk))
+    print("Endpoint: {}".format(self._local_config.endpoint))
 
 def main():
   fire.Fire(FDSCli)
